function [Kc,Kf,Q]=cor2obr(plant,SYS_Kr,tol)
%
% Observer-Based Realization of a given controller
%================================================================
%
% [Kc,Kf,Q] = COR2OBR(PLANT,SYS_K) compute a real Observer Based 
% Realization, that is the Youla parameterization (defined by Kc, 
% Kf and Q), of a given controller SYS_K for a given plant
% PLANT in the case:    NK (SYS_K order) >= N (PLANT order). 
%
% In the discrete-time case PLANT and SYS_K must have
% the same sampling period. Then, the function computes the 
% predictor observer-based realization.
%
% Remarks: * SYS_K, PLANT and Q are defined as SS object,
%          * feedback between PLANT and SYS_K is positive,
%          * a real solution may not exit,
%          * NQ (order of Q) = NK - N.
% 
% This function plots the map of closed-loop eigenvalues (red x)
% and PLANT open-loop eigenvalues (blue +) in the complex plane. 
% Then, the user can choose, in a interactive procedure, the 
% closed-loop  eigenvalues distribution between:
%  * state feedback dynamics [A-BKc] (blue o),
%  * state estimation dynamics [A-KfC] (red o),
%  * Youla parameter dynamics (Q)  (green o).
%
% Uncontrollable eigenvalues are automatically assigned to [A-BKc].
% Unobservable eigenvalues are automatically assigned to [A-KfC].
% (the controller SYS_K is assumed to be minimal).
% Auto-conjugate eigenvalues are assigned together.
%
% [KC,KF,Q] = COR2OBR(PLANT,SYS_K,TOL) allows a tolerance TOL
% (default: 10^-6) to be taken into account in the unobservable
% and uncontrollable subspaces computation.
% 
% OBR = COR2OBR(...) creates a structure variable OBR with
% fields:
%   * OBR.mod : the model of the plant (=PLANT),
%   * OBR.Kc  : state feedback gain,
%   * OBR.Kf  : state estimator gain,
%   * OBR.Q   : Youla parameter (ss),
%   * OBR.obr : the observer-based realisation (ss) of the
%               controller,
%   * OBR.M   : the transformation matrix between the old and the
%               new state space realizations of the controller:
%                   X_k = M [X_hat X_Q].
%
% Reference: D. Alazard and P. Apkarian "Exact observer based 
% structures  for arbitrary compensators" International Journal 
% of Robust and Non-Linear Control, 1999, Vol.9, pp 101-118. 
%
% See also: COR2OBRA, OBR2COR, COR2TFG, COR2TFGA

% D. Alazard 01/99
% revised: 11/06
% revised: 11/10 to handle SS objects

% Closed-loop matrix computation:

if nargin==2, tol=10^-6;end;
[ac,bc,cc,dc,Ts]=ssdata(SYS_Kr);
[A,B2,C2,D22,Ts]=ssdata(plant);
n=size(A,1);nk=size(ac,1);ntot=n+nk;
if nk<n, 
    disp('Controller order lower than plant model order: no solution !!');
    disp('Use: cor2tfg');
    Kc=[];Kf=[];Q=[];M=[];
    return;
end
[acd,bcd,ccd,dcd]=feedback(ac,bc,cc,dc,[],[],[],-D22);
[abf,bbf,cbf,dbf]=feedback(A,B2,C2,0*D22,acd,bcd,ccd,dcd,1);

% ===================================================
% Closed-loop and open loop eigen values map 
% ===================================================
[V,D]=eig(abf);
lesvp=diag(D);

figure
plot(real(lesvp),imag(lesvp),'rx','LineWidth',2);
hold on
vpG=eig(A);
plot(real(vpG),imag(vpG),'b+','LineWidth',2);
if Ts==0,
    sgrid
else
    zgrid
end
zoom on
% ==================================================
% Interactive selection
% ==================================================
nbf=length(lesvp);
nbo=size(A,1);
index1=[];
index2=[];
reste=[];
ii=1;
while ii<=nbf,
  valp=lesvp(ii);
  if imag(valp)==0;
      ir=ii;ii=ii+1;
  else
      ir=[ii ii+1];ii=ii+2;
  end
  if rank([A-valp*eye(nbo,nbo) B2],tol)<nbo,
      index1=[ir index1];
      if length(index1)>nbo, disp('No solution !!!');return;end;
      plot(real(lesvp(ir)),imag(lesvp(ir)),'bo','LineWidth',2);
  elseif rank([A'-valp*eye(nbo,nbo) C2'],tol)<nbo,
      index2=[index2 ir];
      if length(index2)>nbo, disp('No solution !!!');return;end;
      plot(real(lesvp(ir)),imag(lesvp(ir)),'ro','LineWidth',2);
  else
      reste=[reste ir];
  end
end

% Faisaibility of a real parameterization:
nGbar=length(index1);
nObar=length(index2);
npreal=find(imag(lesvp(reste))==0);
vpreal_req=(mod(n-nGbar,2)~=0)+(mod(n-nObar,2)~=0)+(mod(nk-n,2)~=0);
if length(npreal)<vpreal_req,
         disp('No real solution !!');
         Kc=[];Kf=[];Q=[];M=[];
         return
end
index1s=index1;
index2s=index2;
index3=[];
ii=1;
while ii<=length(reste),
   valp=lesvp(reste(ii));
   if imag(valp)==0;
      ir=reste(ii);ii=ii+1;
   else
      ir=[reste(ii) reste(ii+1)];ii=ii+2;
   end
   ok=0;
   alarm=0;
   while ~ok
      if alarm==0,
         text=['You must assign ',num2str(n-length(index1)),', ',...
            num2str(n-length(index2)),' and ',num2str(nk-n-length(index3)),' ,',...
           ' real or auto-conjugate eigenvalue(s) to A-BKc, A-KfC, Q, respectively'];
         menu(text,'Ok');
      end
      text=['Where do you want to assign: ',...
             num2str(valp),' ?'];
      choix=menu(text,'A-BKc','A-KfC','Q','restart procedure');
      switch choix,
      case 1, 
         if length(index1)==n,
             menu('A-BKc dynamics is fulfilled, choose another one !','Ok');
             alarm=alarm+1;
         elseif length(index1)+length(ir)>n,
             if alarm>2,
                 text='Your previous assignments lead to complex parameters.';
                 text=[text;'You must restart the procedure !                     '];
                 menu(text,'Ok');
             else
                menu('Only a real eigenvalue can be now assigned to A-BKc!!','Ok');
                alarm=alarm+1;
             end
         else
             ok=1;index1=[index1 ir];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'bo','LineWidth',2);
             if (length(index1)==n)&(length(index2)==n),
                 menu('Selection is completed','Ok');
                 index3=[reste(ii:length(reste)) index3];ii=length(reste)+1;
                 plot(real(lesvp(index3)),imag(lesvp(index3)),'go','LineWidth',2);
             end
             if (length(index1)==n)&(length(index3)==nk-n),
                 menu('Selection is completed','Ok');
                 index2=[reste(ii:length(reste)) index2];ii=length(reste)+1;
                 plot(real(lesvp(index2)),imag(lesvp(index2)),'ro','LineWidth',2);
             end
         end
      case 2,
         if length(index2)==n,
             menu('A-KfC dynamics is fulfilled, choose another one !','Ok');
             alarm=alarm+1;
         elseif length(index2)+length(ir)>n,
             if alarm>2,
                 text='Your previous assignments lead to complex parameters.';
                 text=[text;'You must restart the procedure !                     '];
                 menu(text,'Ok');
             else
                menu('Only a real eigenvalue can be now assigned to A-KfC!','Ok');
                alarm=alarm+1;
             end
         else
             ok=1;index2=[ir index2];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'ro','LineWidth',2);
             if (length(index2)==n)&(length(index1)==n),
                 menu('Selection is completed','Ok');
                 index3=[reste(ii:length(reste)) index3];ii=length(reste)+1;
                 plot(real(lesvp(index3)),imag(lesvp(index3)),'go','LineWidth',2);
             end
             if (length(index2)==n)&(length(index3)==nk-n),
                 menu('Selection is completed','Ok');
                 index1=[reste(ii:length(reste)) index1];ii=length(reste)+1;
                 plot(real(lesvp(index1)),imag(lesvp(index1)),'bo','LineWidth',2);
             end
        end
      case 3,
         if length(index3)==nk-n,
             menu('Q(s) dynamics is fulfilled, choose another one !','Ok');
             alarm=alarm+1;
         elseif length(index3)+length(ir)>nbf-nbo-nbo,
             if alarm>2,
                 text='Your previous assignments lead to complex parameters.';
                 text=[text;'You must restart the procedure !                     '];
                 menu(text,'Ok');
             else
                menu('Only a real eigenvalue can be now assigned to Q(s) !','Ok');
                alarm=alarm+1;
             end
         else
             ok=1;index3=[index3 ir];
             plot(real(lesvp(ir)),imag(lesvp(ir)),'go','LineWidth',2);
             if (length(index3)==nk-n)&(length(index1)==n),
                 menu('Selection is completed','Ok');
                 index2=[reste(ii:length(reste)) index2];ii=length(reste)+1;
                 plot(real(lesvp(index2)),imag(lesvp(index2)),'ro','LineWidth',2);
             end
             if (length(index3)==nk-n)&(length(index2)==n),
                 menu('Selection is completed','Ok');
                 index1=[reste(ii:length(reste)) index1];ii=length(reste)+1;
                 plot(real(lesvp(index1)),imag(lesvp(index1)),'bo','LineWidth',2);
             end
         end
      case 4,
         ok=1;index1=index1s;index2=index2s;index3=[];ii=1;
         clf
         plot(real(lesvp),imag(lesvp),'rx','LineWidth',2);
         hold on
         plot(real(vpG),imag(vpG),'b+','LineWidth',2);
         sgrid
         plot(real(lesvp(index1)),imag(lesvp(index1)),'bo','LineWidth',2);
         plot(real(lesvp(index2)),imag(lesvp(index2)),'ro','LineWidth',2);
      end
   end
end
%menu('Selection is completed','Ok');
index132=[index1 index3 index2];
M=V(:,index132);
D=D(index132,index132);
[M,D]=cdf2rdf(M,D);

% ==================================================
% Oberver-based parameter computation
% ==================================================


T=real(M(nbo+1:nbf,1:nbo)*inv(M(1:nbo,1:nbo)));

F=acd-T*B2*ccd;
G=bcd-T*B2*dcd;

U=real(T*M(1:nbo,n+1:nbf)-M(nbo+1:nbf,nbo+1:nbf));
U1=U(:,1:nbf-2*nbo);
Um1=inv(U);
U1t=Um1(1:nbf-2*nbo,:);
U2t=Um1(nbf-2*nbo+1:nbf-nbo,:);

Gtilde1=U1t*G;
Gtilde2=U2t*G;
Ttilde1=U1t*T;
Ttilde2=U2t*T;

Kf=inv(Ttilde2)*Gtilde2;
Kc=-ccd*T-dcd*C2;
AQ=real(D(nbo+1:nbf-nbo,nbo+1:nbf-nbo));
BQ=Gtilde1-Ttilde1*inv(Ttilde2)*Gtilde2;
CQ=ccd*U1;
DQ=dcd;

Q=ss(AQ,BQ,CQ,DQ,Ts);
  
% matrice de passage finale:
M=[T U1];

if nargout==1,
    temp.mod=plant;
    temp.Kc=Kc;
    temp.Kf=Kf;
    temp.Q=Q;
    temp.obr=obr2cor(plant,Kc,Kf,Q);
    temp.M=M;
    Kc=temp;
end


