function [Kc,Kf,Q]=cor2obra(onboard,correcteur,nbpoints,tol)
%
% Observer-Based Realization of a given controller
%================================================================
%
% [Kc,Kf,Q] = COR2OBRA(PLANT,SYS_K) compute a real Observer Based 
% Realization, that is the Youla parameterization (defined by Kc, 
% Kf and Q), of a given controller SYS_K for a given plant
% PLANT in the case:    NK (SYS_K order) >= N (PLANT order). 
%
% In the discrete-time case PLANT and SYS_K must have
% the same sampling period. Then, the function computes the 
% predictor observer-based realization.
%
% Remarks: * SYS_K, PLANT and Q are defined as SS object,
%          * feedback between PLANT and SYS_K is positive,
%          * a real solution may not exit,
%          * NQ (order of Q) = NK - N.
%
% Among the combinatory set of solutions (Kc,Kf,Q) a particular 
% solution is selected. This solution consists to assign to the 
% state feedback dynamics the N closed loop eigenvalues located 
% on root loci starting from the N plant open-loop eigenvalues. 
% (This root locus considers the same loop gain varying from 0 
% to 1 on all control channels.)
% The NK other closed-loop eigenvalues are assigned to A-KfC
% and to the Youla parameter Q dynamics.
% The proposed assignment can be plotted in the complex plane
% using function OBRMAP.
%
% Uncontrollable eigenvalues are automatically assigned to [A-BKc].
% Unobservable eigenvalues are automatically assigned to [A-KfC].
% (the controller SYS_K is assumed to be minimal).
% Auto-conjugate eigenvalues are assigned together.
%
% [KC,KF,Q] = COR2OBRA(SYS,COR,NBPOINTS) allows to fix the number
% of intermediate points on root loci (200 by default).
%
% [KC,KF,Q] = COR2OBRA(PLANT,SYS_K,NBPOINTS,TOL) allows a 
% tolerance TOL (default: 10^-6) to be taken into account in the
% unobservable and uncontrollable subspaces computation.
%
% OBR = COR2OBRA(...) creates a structure variable OBR with
% fields:
%   * OBR.mod : the model of the plant (=PLANT),
%   * OBR.Kc  : state feedback gain,
%   * OBR.Kf  : state estimator gain,
%   * OBR.Q   : Youla parameter (ss),
%   * OBR.obr : the observer-based realisation (ss) of the
%               controller,
%   * OBR.M   : the transformation matrix between the old and the
%               new state space realizations of the controller:
%                   X_k = M [X_hat X_Q].
%
% See also: COR2OBR, OBR2COR, COR2TFG, COR2TFGA, OBRMAP

%  	Delmond Fabien, 04/2002
%   Revised by D. Alazard 03/06 and 12/2010


if nargin==2, tol=10^-6; nbpoints=200; end
if nargin==3, tol=10^-6; end


auto=0;


% Data
[Ac,Bc,Cc,Dc,Ts]=ssdata(correcteur);
[As,Bs,Cs,Ds,Ts]=ssdata(onboard);
Ns=size(As,1);
modele=ss(As,Bs,Cs,Ds,Ts);
correcteur=ss(Ac,Bc,Cc,Dc,Ts);
Nc=size(Ac,1);
if Nc<Ns, 
    disp('Controller order lower than plant model order: no solution !!');
    Kc=[];Kf=[];Q=[];M=[];
    return;
end
N=Nc+Ns;
Pc=size(Dc,1);
reto=ss([],[],[],-Ds);
corprim=feedback(correcteur,reto);
Acd=corprim.a; Bcd=corprim.b; Ccd=corprim.c; Dcd=corprim.d;
[Acl,Bcl,Ccl,Dcl]=feedback(As,Bs,Cs,0*Ds,Acd,Bcd,Ccd,Dcd,1);

% ==========================================================
% Incontrollable and inobservable sub-spaces:
[At,Bt,Ct,M,NNO,NNC]=ctrbobsvf(Acl,Bcl,Ccl,tol);
vp_inob=[];
vp_ingo=[];
if NNO, vp_inob=diag(At(1:NNO,1:NNO));end;
if NNC, vp_ingo=diag(At(N-NNC+1:N,N-NNC+1:N));end;


vp=diag(At);

% Faisaibility of a real parameterization:
vpOG=diag(At(NNO+1:N-NNC,NNO+1:N-NNC));
npreal=find(imag(vpOG)==0);
vpreal_req=(mod(Ns-NNC,2)~=0)+(mod(Ns-NNO,2)~=0)+(mod(Nc-Ns,2)~=0);
if length(npreal)<vpreal_req,
         disp('No real solution !!');
         Kc=[];Kf=[];Q=[];M=[];
         return
end

% ==========================================================
% Eigenvalues selection


% Open-loop eigenvalues
vps=eig(As);
vpc=eig(Acd);
vp0=[vps;vpc];


% Marker
mark=[(1:Ns)';-(1:Nc)'];


% Open-loop and closed-loop definition
sysbo=correcteur*modele;
[Ab,Bb,Cb,Db]=ssdata(sysbo);
k=eye(Pc);
[Abf,Bbf,Cbf,Dbf]=ssdata(feedback(sysbo,k,1));


% Tracking eigenvalues from open-loop to closed-loop.
vp=[vp0 vp0];
for ii=1:nbpoints
	kk=ii/nbpoints*k;
	App=Ab+Bb*kk*(eye(Pc)-Db*kk)^-1*Cb;
    eigapp=eig(App);
	vp=[vp(:,2) eigapp];
	mat=vp(:,1)*ones(1,N)-ones(N,1)*conj(vp(:,2)');
	mat=abs(mat);
	corres=[];
	for jj=1:N,
		matvec=mat(:);
		[y,j]=min(matvec);
		qo=fix((j-1)/N);  	% corresponds to column of mat -1
		re=j-N*qo;		% corresponds to row of mat
		corres=[[corres],[qo+1;re]];
		mat(:,qo+1)=10^3*ones(N,1);
		mat(re,:)=10^3*ones(1,N);
	end
	mark(corres(1,:))=mark(corres(2,:));
end



% 3 cases:
%  * Ns=Nc, Q statique
%  * Ns<Nc, Q dynamique (Nc-Ns poles)
%  * Real T is not possible


% Faisaibility of a real parameterization
parNs=Ns-2*fix(Ns/2);
parNsc=(Nc-Ns)-2*fix((Nc-Ns)/2);
nbreels=vp(:,2)-real(vp(:,2));
nbreels=length(find(abs(nbreels)==0));
if nbreels<(2*parNs+parNsc),
    disp('	No real solution !!');
    Kc=[];Kf=[];Q=[];M=[];
	return
end

% 
vp=[vp(:,2) diag(At)];
mat=vp(:,1)*ones(1,N)-ones(N,1)*conj(vp(:,2)');
mat=abs(mat);
[y,j]=min(mat);
mark=mark(j);
mark=mark(:);


%======================================================================
% groups of eigenvalues:

  % First case: Ns>=Nc
  if Ns>=Nc,
	index=zeros(1,N);
	ii=1;
	jj=Ns+1;
	for i=1:N
		if mark(i)>0, index(i)=ii; ii=ii+1;
		else, index(i)=jj; jj=jj+1;
		end
	end
  % Second case, Ns<Nc
  else,
  	% Introduction of a sensitivity index to choose eigenvelues between 
    % A-GC, et Q(s)
  	[vpmat,lambda]=eig(Abf);
  	lambda=diag(lambda);
  	vp=[lambda diag(Abf)];
	mat=vp(:,1)*ones(1,N)-ones(N,1)*conj(vp(:,2)');
	mat=abs(mat);	
  	dlambda=[];
  	vpmatinv=inv(vpmat);
  	for ii=1:N,
  		% Sensitivity of controller eigenvalues
  		dlambda=[dlambda;vpmatinv(ii,:)*Bbf*Cbf*vpmat(:,ii)];
  	end
  	dlambda=abs(dlambda)./abs(lambda);
  	[y,j]=min(mat);
  	dlambda=dlambda(j);
  	lambda=lambda(j);
  	class=[];
  	for i=1:N,
  		 if mark(i)<0, class=[[class],[dlambda(i);i]];
  		 end
  	end
  	[class(1,:),j]=sort(class(1,:));
  	class(2,:)=class(2,j);
  	
  	index=zeros(1,N);
	ii=1;
	jj=Ns+1;
	ll=Nc+1;
	for i=1:N
		if mark(i)>0, index(i)=ii; ii=ii+1;
		elseif (find(class(2,:)==i))<=(Nc-Ns),
			index(i)=jj; jj=jj+1;
		else index(i)=ll; ll=ll+1;
		end
	end
  end    
  	


  % Ordered Schurd decomposition:
  [M,At]=schord(M,At,index);
  vp=diag(At);
  Up=M; Tp=At;
 
  % Check that the minimal real eigenvalue for (A-KfC) and AQ is reached.
  ind_reel=find(abs(imag(vp))<10^-8);  
  ind_complex=find(abs(imag(vp))>10^-8);
  vp_r_f=find(ind_reel>Ns);
  index=(1:N);
  if length(vp_r_f)<parNs+parNsc,
      bid=find(ind_reel<Ns+1);
      ind_reel=ind_reel(bid);
      [~,ind]=min(real((ind_reel)));
      ind0=ind_reel(ind);
      bid=find(ind_complex>Ns);
      ind_complex=ind_complex(bid);
      [~,ind]=max(real((ind_complex)));
      ind1=ind_complex(ind);
      index([ind0,ind1])=index([ind1,ind0]);
  end
  [Up,Tp]=schord(Up,Tp,index);
  vp=diag(Tp);
 
  
  % Bifurcation problem:
  probleme=[];
  mat=abs(vp(1:Ns)*ones(1,Ns)-ones(Ns,1)*vp(1:Ns)');
  probleme=min(mat);
  counter=0;
  while ((probleme*ones(Ns,1))>10^-8*Ns)&(counter<200),
    counter=counter+1;
  	vps=vp(1:Ns);
  	vpc=vp(Ns+1:N);
%  	[vp_reel,ind_reel]=find(abs(imag(vp))<10^-8);
  	ind_reel=find(abs(imag(vp))<10^-8);    
  	vp_reel=real(vp(ind_reel));
  	index=(1:N);
  	[pb,ind]=max(probleme);
  	probleme(ind)=0;
  	if (probleme*ones(Ns,1))>10^-8*Ns,  
  	   [pb,ind1]=max(probleme);
  	   % Choice of the pair assigned to  A-B.Kc
  	   if real(vp(ind))>real(vp(ind1)),
  	      valfind=vp(ind)';
	      [pb,ind]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
	      index([ind1,ind+Ns])=index([ind+Ns,ind1]);
  	   else,
  	      valfind=vp(ind1)';
	      [pb,ind1]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
	      index([ind,ind1+Ns])=index([ind1+Ns,ind]);
	   end
	elseif ~isempty(find(ind_reel>Ns)), % real eigenvalues in estimation
	   if isempty(find(ind_reel<=Ns)), 
	      [y,jj]=max(vp_reel);
	      ind1=ind_reel(jj);
	      index([ind1,ind])=index([ind,ind1]);
	   else,
	      indp=[ind_reel(find(ind_reel<=Ns))'];
	      choix=real(vp(indp));
	      [y,jj]=sort(choix);
	      ind1=indp(jj(1));  
	      if ind1~=ind, 
	         valfind=vp(ind)';
	         [pb,ind]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
	         index([ind+Ns,ind1])=index([ind1,ind+Ns]);
	      else, 
	         indp=ind_reel(find(ind_reel>Ns));
	         choix=real(vp(indp));
	         [y,jj]=sort(choix);
	         ind1=indp(jj(length(jj)));
	         index([ind1,ind])=index([ind,ind1]);
	      end
	   end  	
  	else,
  	   indp=[ind_reel(find(ind_reel<=Ns))];
  	   choix=abs(real(vp(indp)));
  	   [y,jj]=sort(choix);
  	   ind1=indp(jj(length(jj)));
  	   valfind=vp(ind)';
	   [pb,ind]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
  	   index([ind+Ns,ind1])=index([ind1,ind+Ns]);
  	end
  	[Up,Tp]=schord(Up,Tp,index);
	vp=diag(Tp);
	mat=abs(vp(1:Ns)*ones(1,Ns)-ones(Ns,1)*vp(1:Ns)');
  	probleme=min(mat);
  	 
  end
  if counter==200,
      clc
      disp('Problem to split eigenvalues:')
      disp('* solution may be inaccurate!!');
      disp('* try to reduce TOL or to increase NBPOINTS.');
  end
  
  % Autoconjugate pairs splitted in Q and (A-KfC)
  mat=abs(vp(Ns+1:Nc)*ones(1,Nc-Ns)-ones(Nc-Ns,1)*vp(Ns+1:Nc)');
  probleme=min(mat);
  while and(((probleme*ones(Nc-Ns,1))>=10^-8*(Nc-Ns)),Ns~=Nc),
	vpq=vp(Ns+1:Nc);
	index=(1:N);	
    [pb,ind0]=max(probleme);
	probleme(ind0)=0;
    valfind=vpq(ind0)';
    [pb,ind]=min(abs(valfind*ones(N-Nc,1)-vp(Nc+1:N)));
    if (probleme*ones(Nc-Ns,1))>10^-8*(Nc-Ns),
      [pb,ind1]=max(probleme);
      index([ind1+Ns,ind+Nc])=index([ind+Nc,ind1+Ns]);
    else,
      [y,ind1]=min(diag(mat));
      if y<10^-8,  
          index([ind1+Ns,ind+Nc])=index([ind+Nc,ind1+Ns]);
      else % change with a real eigenvalue
          ind_reel=find(abs(imag(vp(Nc+1:N)))<10^-8);    
          vp_reel=real(vp(Nc+ind_reel));
          [~,ind1]=min(vp_reel);
          ind1=ind_reel(ind1);
          index([ind0+Ns,ind1+Nc])=index([ind1+Nc,ind0+Ns]);
      end
    end
  	[Up,Tp]=schord(Up,Tp,index);
	vp=diag(Tp);
	mat=abs(vp(Ns+1:Nc)*ones(1,Nc-Ns)-ones(Nc-Ns,1)*vp(Ns+1:Nc)');
  	probleme=min(mat);
  end	
  M=Up;
  At=Tp;


  % Real representation:
  qord=[real(M),imag(M)];
  [M,R]=qr(qord);
  At=M'*Acl*M;At=real(At);
  M=real(M);


% ==================================================
% Oberver-based parameter computation
% ==================================================


  T=M(Ns+1:N,1:Ns)*inv(M(1:Ns,1:Ns));
  T=real(T);
  Jm=inv(eye(size(Dc*Ds))-Dc*Ds);
  Jp=inv(eye(size(Ds*Dc))-Ds*Dc);
  G=(Bc-T*Bs*Dc)*Jp;
  F=Ac+(Bc*Ds-T*Bs)*Jm*Cc;

U=real(T*M(1:Ns,Ns+1:N)-M(Ns+1:N,Ns+1:N));
U1=U(:,1:N-2*Ns);
Um1=inv(U);
U1t=Um1(1:N-2*Ns,:);
U2t=Um1(N-2*Ns+1:N-Ns,:);

Gtilde1=U1t*G;
Gtilde2=U2t*G;
Ttilde1=U1t*T;
Ttilde2=U2t*T;

Kf=inv(Ttilde2)*Gtilde2;
Kc=-Jm*(Cc*T+Dc*Cs);
AQ=real(At(Ns+1:Nc,Ns+1:Nc));
BQ=Gtilde1-Ttilde1*inv(Ttilde2)*Gtilde2;
CQ=Jm*Cc*U1;
DQ=Jm*Dc;

Q=ss(AQ,BQ,CQ,DQ,Ts);
  
% matrice de passage finale:
M=[T U1];

if nargout==1,
    temp.mod=onboard;
    temp.Kc=Kc;
    temp.Kf=Kf;
    temp.Q=Q;
    temp.obr=obr2cor(onboard,Kc,Kf,Q);
    temp.M=M;
    Kc=temp;
end

  