function [T,F,G]=cor2tfga(onboard,correcteur,nbpoints,tol)
%
% Observer-Based Realization of a given controller
%==========================================================
%
% [T,F,G] = COR2TFGA(PLANT,SYS_K) 
% Given a nk order controller SYS_K defined by a state space 
% realization (Ak, Bk, Ck, Dk) associated with state variable Xk,
% given a n order given plant PLANT defined by a state space 
% realization (A,B,C,D) associated with state variable X,
% COR2TGFA computes the matrix T (nk x n) of the linear combination 
% Z=TX of the plant states X which are estimated by the controller 
% state Xk; that is Xk=Z_hat.
% F and G are the dynamic matrix and the input matrix of the 
% Luenberger observer associated with the controller, that is:
% let u and y be the input and the output of the plant, the 
% controller SYS_K can be parametrized in the following way:
%
%   (s or z)X_k = F X_k + G (y-Du) + T B u
%             u = Ck X_k + Dk y
%
% Remarks: * SYS_K, PLANT are defined as SS objects,
%          * feedback between PLANT and SYS_K is positive,
%          * continuous or discrete-time cases are supported,
%          * a real solution may not exit,
%
% Among the combinatory set of solutions T, F, G, a particular 
% solution is selected. This solution consists to isolate the 
% n closed loop eigenvalues located on root loci starting from 
% the n plant open-loop eigenvalues. The nk other closed-loop 
% eigenvalues are then assigned to F (and thus does not include
% plant uncontrollable eigenvalues).
% (This root locus considers the same loop gain varying from 0 
% to 1 on all control channels.)
%
% The controller SYS_K is assumed to be minimal.
% Auto-conjugate eigenvalues are assigned together.
%
% [T,F,G] = COR2TFGA(SYS,COR,NBPOINTS) allows to fix the number
% of intermediate points on root loci (200 by default).
%
% [T,F,G] = COR2TFGA(PLANT,SYS_K,NBPOINTS,TOL) allows a 
% tolerance TOL (default: 10^-6) to be taken into account in the
% uncontrollable subspace computation.
%
% See also: COR2TFG, COR2OBR, COR2OBRA, OBR2COR, OBRMAP

%  	Delmond Fabien, 04/2002
%   Revised by D. Alazard 03/06 and 12/2010


if nargin==2, tol=10^-6; nbpoints=200; end
if nargin==3, tol=10^-6; end

auto=0;


% Data
[Ac,Bc,Cc,Dc]=ssdata(correcteur);
[As,Bs,Cs,Ds]=ssdata(onboard);
Ns=size(As,1);
modele=ss(As,Bs,Cs,Ds);
correcteur=ss(Ac,Bc,Cc,Dc);
Nc=size(Ac,1);
N=Nc+Ns;
Pc=size(Dc,1);
reto=ss([],[],[],-Ds);
corprim=feedback(correcteur,reto);
Acd=corprim.a; Bcd=corprim.b; Ccd=corprim.c; Dcd=corprim.d;
[Acl,Bcl,Ccl,Dcl]=feedback(As,Bs,Cs,0*Ds,Acd,Bcd,Ccd,Dcd,1);

% ==========================================================
% Incontrollable and inobservable sub-spaces:
[At,Bt,Ct,M,NNO,NNC]=ctrbobsvf(Acl,Bcl,Ccl,tol);
vp_inob=[];
vp_ingo=[];
if NNO, vp_inob=diag(At(1:NNO,1:NNO));end;
if NNC, vp_ingo=diag(At(N-NNC+1:N,N-NNC+1:N));end;


vp=diag(At);

% Faisaibility of a real parameterization
if (mod(Ns-NNC,2)~=0),  % case: gouvernable plant order is odd
    if (length(find(imag(vp(1:N-NNC))==0))==0),  %  ===> no solution
        disp('No real solution !!');
        T=[];F=[];G=[];
        return
    end
end
% ==========================================================
% Eigenvalues selection


% Open-loop eigenvalues
vps=eig(As);
vpc=eig(Acd);
vp0=[vps;vpc];


% Marker
mark=[(1:Ns)';-(1:Nc)'];


% Open-loop and closed-loop definition
sysbo=correcteur*modele;
[Ab,Bb,Cb,Db]=ssdata(sysbo);
k=eye(Pc);
[Abf,Bbf,Cbf,Dbf]=ssdata(feedback(sysbo,k,1));


% Tracking eigenvalues from open-loop to closed-loop.
vp=[vp0 vp0];
for ii=1:nbpoints
	kk=ii/nbpoints*k;
	App=Ab+Bb*kk*(eye(Pc)-Db*kk)^-1*Cb;
    eigapp=eig(App);
	vp=[vp(:,2) eigapp];
	mat=vp(:,1)*ones(1,N)-ones(N,1)*conj(vp(:,2)');
	mat=abs(mat);
	corres=[];
	for jj=1:N,
		matvec=mat(:);
		[y,j]=min(matvec);
		qo=fix((j-1)/N);  	% corresponds to column of mat -1
		re=j-N*qo;		% corresponds to row of mat
		corres=[[corres],[qo+1;re]];
		mat(:,qo+1)=10^3*ones(N,1);
		mat(re,:)=10^3*ones(1,N);
	end
	mark(corres(1,:))=mark(corres(2,:));
end



% 3 cases:
%  * Ns=Nc, Q statique
%  * Ns<Nc, Q dynamique (Nc-Ns poles)
%  * Real T is not possible


% 
vp=[vp(:,2) diag(At)];
mat=vp(:,1)*ones(1,N)-ones(N,1)*conj(vp(:,2)');
mat=abs(mat);
[y,j]=min(mat);
mark=mark(j);
mark=mark(:);


%======================================================================
% groups of eigenvalues:

  % First case: Ns>=Nc
  if Ns>=Nc,
	index=zeros(1,N);
	ii=1;
	jj=Ns+1;
	for i=1:N
		if mark(i)>0, index(i)=ii; ii=ii+1;
		else, index(i)=jj; jj=jj+1;
		end
	end
  % Second case, Ns<Nc
  else,
  	% Introduction od a sensitivity index to choose eigenvelues between 
    % A-GC, et Q(s)
  	[vpmat,lambda]=eig(Abf);
  	lambda=diag(lambda);
  	vp=[lambda diag(Abf)];
	mat=vp(:,1)*ones(1,N)-ones(N,1)*conj(vp(:,2)');
	mat=abs(mat);	
  	dlambda=[];
  	vpmatinv=inv(vpmat);
  	for ii=1:N,
  		% Sensitivity of controller eigenvalues
  		dlambda=[dlambda;vpmatinv(ii,:)*Bbf*Cbf*vpmat(:,ii)];
  	end
  	dlambda=abs(dlambda)./abs(lambda);
  	[y,j]=min(mat);
  	dlambda=dlambda(j);
  	lambda=lambda(j);
  	class=[];
  	for i=1:N,
  		 if mark(i)<0, class=[[class],[dlambda(i);i]];
  		 end
  	end
  	[class(1,:),j]=sort(class(1,:));
  	class(2,:)=class(2,j);
  	
  	index=zeros(1,N);
	ii=1;
	jj=Ns+1;
	ll=Nc+1;
	for i=1:N
		if mark(i)>0, index(i)=ii; ii=ii+1;
		elseif (find(class(2,:)==i))<=(Nc-Ns),
			index(i)=jj; jj=jj+1;
		else index(i)=ll; ll=ll+1;
		end
	end
  end    
  	


  % Ordered Schurd decomposition:
  [M,At]=schord(M,At,index);
  vp=diag(At);
  Up=M; Tp=At;
 
 
  % Bifurcation problem:
  probleme=[];
  mat=abs(vp(1:Ns)*ones(1,Ns)-ones(Ns,1)*vp(1:Ns)');
  probleme=min(mat);
  counter=0;
  while ((probleme*ones(Ns,1))>10^-8*Ns)&(counter<200),
    counter=counter+1;
 	vps=vp(1:Ns);
  	vpc=vp(Ns+1:N);
%  	[vp_reel,ind_reel]=find(abs(imag(vp))<10^-8);
  	ind_reel=find(abs(imag(vp))<10^-8);    
  	vp_reel=real(vp(ind_reel));
  	index=(1:N);
  	[pb,ind]=max(probleme);
  	probleme(ind)=0;
  	if (probleme*ones(Ns,1))>10^-8*Ns,  
  	   [pb,ind1]=max(probleme);
  	   % Choice of the pair assigned to  A-B.Kc
  	   if real(vp(ind))>real(vp(ind1)),
  	      valfind=vp(ind)';
	      [pb,ind]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
	      index([ind1,ind+Ns])=index([ind+Ns,ind1]);
  	   else,
  	      valfind=vp(ind1)';
	      [pb,ind1]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
	      index([ind,ind1+Ns])=index([ind1+Ns,ind]);
	   end
	elseif isempty(find(ind_reel>Ns))==0, % real eigenvalues in estimation
	   if isempty(find(ind_reel<=Ns))==1, 
	      [y,jj]=max(vp_reel);
	      ind1=ind_reel(jj);
	      index([ind1,ind])=index([ind,ind1]);
	   else,
	      indp=[ind,ind_reel(find(ind_reel<=Ns))'];
	      choix=real(vp(indp));
	      [y,jj]=sort(choix);
	      ind1=indp(jj(1));  
	      if ind1~=ind, 
	         valfind=vp(ind)';
	         [pb,ind]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
	         index([ind+Ns,ind1])=index([ind1,ind+Ns]);
	      else, 
	         indp=ind_reel(find(ind_reel>Ns));
	         choix=real(vp(indp));
	         [y,jj]=sort(choix);
	         ind1=indp(jj(length(jj)));
	         index([ind1,ind])=index([ind,ind1]);
	      end
	   end  	
  	else,
  	   indp=[ind_reel(find(ind_reel<=Ns))];
  	   choix=abs(real(vp(indp)));
  	   [y,jj]=sort(choix);
  	   ind1=indp(jj(length(jj)));
  	   valfind=vp(ind)';
	   [pb,ind]=min(abs(valfind*ones(Nc,1)-vp(Ns+1:N)));
  	   index([ind+Ns,ind1])=index([ind1,ind+Ns]);
  	end
  	[Up,Tp]=schord(Up,Tp,index);
	vp=diag(Tp);
	mat=abs(vp(1:Ns)*ones(1,Ns)-ones(Ns,1)*vp(1:Ns)');
  	probleme=min(mat);
  	 
  end
  if counter==200,
      clc
      disp('Problem to split eigenvalues:')
      disp('* solution may be inaccurate!!');
      disp('* try to reduce TOL or to increase NBPOINTS.');
  end

  M=Up;	% back to initial variable
  At=Tp;


  % Real representation:
  qord=[real(M),imag(M)];
  [M,R]=qr(qord);
  At=M'*Acl*M;At=real(At);
  M=real(M);


  % Change of variable matrices: T, U
  T=M(Ns+1:N,1:Ns)*inv(M(1:Ns,1:Ns));
  T=real(T);
  Jm=inv(eye(size(Dc*Ds))-Dc*Ds);
  Jp=inv(eye(size(Ds*Dc))-Ds*Dc);
  G=(Bc-T*Bs*Dc)*Jp;
  F=Ac+(Bc*Ds-T*Bs)*Jm*Cc;
  