function [OBR]=obcanon(plant)
%
% Observer-Based Realization of a given system on a canonic
% system (with same order).
%================================================================
%
% OBR = OBCANON(PLANT) compute a real Observer Based Realization,
% that is the Youla parameterization (defined by OBR.Kc, OBR.Kf 
% and OBR.Q), of a given (continuous or discrete time) system
% PLANT on a canonical system provided in OBR.mod 
%
% OBR is a structured variable with several fields.
%
% The canonical system OBR.mod is defined by:
%   OBR.mod=ss([zeros(n,1) [eye(n-1);zeros(1,n-1)]],...
%       [zeros(n-p,p);eye(p)],[eye(m) zeros(m,n-m)],zeros(m,p));
%   where n, m, p are the order, the number of imputs and the 
%   number of outputs of PLANT. That is: OBR.mod (mxp) is a set 
%   of n integrators (delays) in series with:
%          * the inputs of the first p intergrators (delays) are 
%            are connected to the p inputs
%          * the ouputs of the last m intregrators (delays) are
%            connected to the m ouputs.
%
% Remarks: * A minimal state-space realization of PLANT is first
%            computed.
%          * Auto-conjugate eigenvalues are assigned together.
%
% OBR.obr is the new state space realization of PLANT
% OBR.M is the transformation matrix between the old and the
%       new state space vector of the system:
%                   X_plant = M [X_hat]
%
% See also: OBR2COR, COR2TFG, COR2TFGA, COR2OBR, COR2OBRA

% D. Alazard 01/99
% revised: 11/06
% revised: 11/10 to handle SS objects

% Closed-loop matrix computation:

plant=ss(plant);
plant=minreal(plant);
[a,b,c,d,Ts]=ssdata(plant);
n=size(plant.a,1);
[p,m]=size(plant.d);
SYSO=ss([zeros(n,1) [eye(n-1);zeros(1,n-1)]],[zeros(n-p,p);eye(p)],...
        [eye(m) zeros(m,n-m)],zeros(m,p),Ts);
[T,F,G]=cor2tfga(SYSO,plant);
Kc=-(plant.c*T+plant.d*SYSO.c);
Kf=inv(T)*plant.b-SYSO.b*plant.d;
Q=plant.d;
OBR.mod=SYSO;
OBR.Kc=Kc;
OBR.Kf=Kf;
OBR.Q=Q;
OBR.obr=obr2cor(SYSO,Kc,Kf,Q);
OBR.M=T;
