function [SYS_K]=obr2cor(PLANT,K,G,Q)
%
% SYS_K=OBR2COR(PLANT,K,G,Q)
% compute the state space realization SYS_K=ss(AK,BK,CK,DK) 
% of the observer-based controller defined by the 
% Youla parameterization:
%    - K: state feedback gain, 
%    - G: state estimation gain, 
%    - Q: dynamic Youla parameter 
% This parameterization concern the plant defined by PLANT.
%
% PLANT,Q and SYS_K are defined as SS objects 
% (continous and discrete time cases are considered).
%
% Let us note: [A,B,C,D,Ts]=ssdata(PLANT), then the controller 
% realization is:
%                 ________________________
%                |                        |
%                | A-BK-GC+GDK | G | B-GD | 
%         u <----|     -K      | 0 |   I  |<----- y
%    y-yhat /----|    -C+DK    | I |  -D  |<---\  e
%           |    |________________________|    |
%           |                                  |
%           |               ___________        |
%           |              |           |       |
%           \------------->| Q(s or z) |-------/
%                          |___________|
%
% Remark: positive feedback is assumed for SYS_K.
%
% Reference: D. Alazard and P. Apkarian "Exact observer based 
% structures for arbitrary compensators" International Journal 
% of Robust and Non-Linear Control, 1999, Vol.9, pp 101-118. 
%
% See also: COR2OBR, COR2OBRA, OBR2COR2DDL

% D. Alazard 01/99
% Revised 11/2010

[A,B,C,D,Ts]=ssdata(PLANT);
[p,m]=size(D);
SYS_K=ss([A-B*K-G*C+G*D*K],[G B-G*D],[-K;-C+D*K],...
         [0*ones(m,p) eye(m,m);eye(p,p) -D],Ts);
SYS_K=lft(SYS_K,Q);



